/*
  This file is part of GerAstro.

  GerAstro is free software: you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
  Foundation, either version 3 of the License, or (at your option) any later
  version.

  GerAstro is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along with
  GerAstro.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef GERPH_CUBICSPLINE_H
#define GERPH_CUBICSPLINE_H


namespace GerPh {


  class CubicSpline {

    public:

      CubicSpline(double aA0, double aA1, double aA2, double aA3,
        double aArgBegin, double aArgRange);
      CubicSpline(
        double aValBegin, double aDrvBegin, double aValEnd, double aDrvEnd,
	int aDrvOrder = 1, double aArgBegin = 0.0, double aArgEnd = 0.0);

      double getValue(double aArg);
      double getValueLinear(double aArg);
      double getDerivative(double aArg);

      double getArgBegin();
      double getArgEnd();
      double getArgRange();
      void setInterval(double aBegin, double aRange);

      double getValBegin();
      double getValEnd();
      double getDrvBegin();
      double getDrvEnd();

      void setBoundaries(
        double aValBegin, double aDrvBegin, double aValEnd, double aDrvEnd,
	int aDrvOrder = 1);
      void setCoefficients(double aA0, double aA1, double aA2, double aA3);

    private:

      double fArgBegin;
      double fArgRange;
      double fRezArgRange;

      double fA0;
      double fA1;
      double fA2;
      double fA3;

  }; // CubicSpline


} // GerPh


#endif
