/*
  This file is part of GerAstro.

  GerAstro is free software: you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
  Foundation, either version 3 of the License, or (at your option) any later
  version.

  GerAstro is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along with
  GerAstro.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef GERPHYS_DEFAULTSPLINEFUNCTION_H
#define GERPHYS_DEFAULTSPLINEFUNCTION_H


#include "cubicspline.h"
#include <gerq/sortedlist.h>


namespace GerPh {


  class DefaultSplineFunction {

    public:

      DefaultSplineFunction(int aCount, double *aArgs, double *aValues,
        double aDrvBegin = 0.0, double aDrvEnd = 0.0, bool aDamped = false);
      DefaultSplineFunction(int aCount, double *aValues,
        double aDrvBegin = 0.0, double aDrvEnd = 0.0, bool aDamped = false);
      virtual ~DefaultSplineFunction();

      double getValue(double aArg);
      double getValueLinear(double aArg);

      int getCountSplines();
      CubicSpline *getSplineAt(int aIndex);
      CubicSpline *getSpline(double aArg);

    private:

      GerQ::SortedList<CubicSpline*> *fSplines;

      void initialize(int aCount, double *aArgs, double *aValues,
        double aDrvBegin = 0.0, double aDrvEnd = 0.0);
      void initDamped(int aCount, double *aArgs, double *aValues,
        double aDrvBegin = 0.0, double aDrvEnd = 0.0);

      static int cmpByStart(CubicSpline *aSpline1, CubicSpline *aSpline2);

  }; // DefaultSplineFunction


} // GerPh


#endif
