/*
  This file is part of GerPhys.

  GerPhys is free software: you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
  Foundation, either version 3 of the License, or (at your option) any later
  version.

  GerPhys is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along with
  GerPhys.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "floatimage.h"


namespace GerPh {

  FloatImage::FloatImage(int aWidth, int aHeight, QImage::Format aFormat):
  QImage(aWidth, aHeight, aFormat) {
    fMap = NULL;
    fDrawing = NULL;
    fDrawArgs = NULL;
  } // FloatImage::FloatImage

  FloatImage::~FloatImage() {
     if (fMap) {
       fMap -> setDevice(NULL);
     } // if
  } // FloatImage::~FloatImage

  void FloatImage::drawMap() {
    if (fMap && !fDrawing) {
      fDrawing = fMap -> getDrawing();
      fDrawArgs = fMap -> getDrawArgs();
    } // if
    if (fDrawing) {
      QPainter painter(this);
      (*fDrawing)(&painter, fDrawArgs);
      painter.end();
    } // if
  } // FloatImage::drawMap

  FloatMap* FloatImage::getMap() {
    return fMap;
  } // FloatImage::getMap

  void FloatImage::setMap(FloatMap *aValue) {
    if (aValue == fMap) {
      return;
    } // if
    if (fMap) {
      fMap -> setDevice(NULL);
    } // if
    fMap = aValue;
    fDrawing = NULL;
    fDrawArgs = NULL;
    if (fMap) {
      fMap -> setDevice(this);
    } // if
  } // FloatImage::setMap

} // GerPh
