/*
  This file is part of GerPhys.

  GerPhys is free software: you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
  Foundation, either version 3 of the License, or (at your option) any later
  version.

  GerPhys is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along with
  GerPhys.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef GERPH_FLOATMAP_H
#define GERPH_FLOATMAP_H


#include "gerphysbase.h"
#include <QBrush>
#include <QFont>
#include <QPaintDevice>
#include <QPainter>


namespace GerPh {


  class FloatMap {

    public:

      FloatMap(QPaintDevice *aDevice);
      virtual ~FloatMap();

      QPaintDevice *getDevice();
      void setDevice(QPaintDevice *aValue);

      void (*getDrawing())(QPainter*, void*);
      void setDrawing(void (*aValue)(QPainter*, void*));
      void *getDrawArgs();
      void setDrawArgs(void *aValue);

      double getXMin();
      double getXMax();
      void setXBorders(double aXMin, double aXMax);

      double getYMin();
      double getYMax();
      void setYBorders(double aYMin, double aYMax);

      double getActualXMin();
      double getActualXMax();
      double getXMiddle();
      double getActualYMin();
      double getActualYMax();
      double getYMiddle();

      double getDiagonal();

      int getXView(double aXModel);
      int getYView(double aYModel);
      int getDeltaXView(double aDeltaX);
      int getDeltaYView(double aDeltaY);
      double getXModel(int aXView);
      double getYModel(int aYView);
      double getDeltaXModel(int aDeltaX);
      double getDeltaYModel(int aDeltaY);

      double getFixRatioXY();
      void setFixRatioXY(double aValue = 1.0);

      static const double NO_FIX_RATIO;

      bool hasFixRatioXY();

      int getMarginLeft();
      int getMarginRight();
      int getMarginTop();
      int getMarginBottom();
      void setMarginLeft(int aValue);
      void setMarginRight(int aValue);
      void setMarginTop(int aValue);
      void setMarginBottom(int aValue);

      Projection *getProjection();
      void setProjection(Projection *aValue, double aRatioXY = 1.0);

      void drawPoint(double aX, double aY, QPainter *aPainter);
      void drawLine(double aXFrom, double aYFrom, double aXTo, double aYTo,
        QPainter *aPainter);
      void drawEllipse(double aX, double aY, double aRX, double aRY,
        QPainter *aPainter);
      void drawEllipseSym(double aX, double aY, int aRX, int aRY,
        QPainter *aPainter);
      void drawSpot(double aX, double aY, double aR, QPainter *aPainter,
        double aStart = 0.0);
      void drawSpotSym(double aX, double aY, int aR, QPainter *aPainter,
        double aStart = 0.0);
      void drawText(double aX, double aY, const QString &aText,
        QPainter *aPainter,
	int aHAlign = ALIGN_CENTER, int aVAlign = ALIGN_CENTER,
	int aHOffset = 0, int aVOffset = 0);

      static void drawSpotSymView(int aX, int aY, int aR, QPainter *aPainter,
        double aStart = 0.0);

      void fillAll(QBrush *aBrush, QPainter *aPainter);
      void fillRect(double aX, double aY, double aDeltaX, double aDeltaY,
        QBrush *aBrush, QPainter *aPainter);

      void getTextSize(const QString &aText, QFont *aFont, QPainter *aPainter,
        int &aWidth, int &aHeight);

      void rescale();

      static const int ALIGN_CENTER;
      static const int ALIGN_LEFT;
      static const int ALIGN_RIGHT;
      static const int ALIGN_TOP;
      static const int ALIGN_BOTTOM;

    private:

      QPaintDevice *fDevice;
      Projection *fProjection;

      void (*fDrawing)(QPainter*, void*);
      void *fDrawArgs;

      double fXMin;
      double fXMax;
      double fYMin;
      double fYMax;

      double fActualXMin;
      double fActualYMax;

      double fFixRatioXY;

      double fDeltaX_W;
      double fW_DeltaX;
      double fDeltaY_H;
      double fH_DeltaY;

      int fMarginLeft;
      int fMarginRight;
      int fMarginTop;
      int fMarginBottom;

  }; // FloatMap


} // GerPh


#endif
