/*
  This file is part of GerPhys.

  GerPhys is free software: you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
  Foundation, either version 3 of the License, or (at your option) any later
  version.

  GerPhys is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along with
  GerPhys.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "vector.h"


#ifndef GERPH_GAUSSIAN_H
#define GERPH_GAUSSIAN_H


namespace GerPh {


  class Gaussian {

    public:

      Gaussian(int aOrder);
      virtual ~Gaussian();

      Vector *newEquation();

      int getOrder();
      int getCountEquations();
      Vector *getEquationAt(int aRow);
      double getValueAt(int aRow);

      void swapRows(int aRow1, int aRow2);
      void addToRow(int aRow1, double aFactor, int aRow2);
      void multRow(int aRow, double aFactor);

      void solve(double aEpsilon = 1.0e-32);

    private:

      int fOrder;
      Vector::List fEquations;

  }; // Gaussian


} // GerPh


#endif
