/*
  This file is part of GerPhys.

  GerPhys is free software: you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
  Foundation, either version 3 of the License, or (at your option) any later
  version.

  GerPhys is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along with
  GerPhys.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef GERPH_R3VECTOR_H
#define GERPH_R3VECTOR_H


namespace GerPh {


  class R3Vector {

    public:

      R3Vector();
      R3Vector(double aX, double aY, double aZ);

      double getAt(int i) const;
      void setAt(int i, double aValue);

      double getX() const;
      double getY() const;
      double getZ() const;
      void setXYZ(double aX, double aY, double aZ);

      double getAbs();

      bool operator== (const R3Vector &v) { return isEqual(v); }
      R3Vector& operator= (const R3Vector &v) { return setEqual(v); }
      R3Vector& operator+= (const R3Vector &v) { return addVec(v); }
      R3Vector& operator-= (const R3Vector &v) { return subVec(v); }
      R3Vector& operator%= (const R3Vector &v) { return crossVec(v); }
      R3Vector& operator*= (double x) { return multDbl(x); }
      R3Vector& operator/= (double x) { return multDbl(1.0 / x); }

    private:

      double fCoeff[3];

      static void checkIdx(int i);

      bool isEqual(const R3Vector &v) const;
      R3Vector& setEqual(const R3Vector &v);
      R3Vector& addVec(const R3Vector &v);
      R3Vector& subVec(const R3Vector &v);
      R3Vector& crossVec(const R3Vector &v);
      R3Vector& multDbl(double x);

  }; // R3Vector


  R3Vector operator+ (const R3Vector &v1, const R3Vector &v2);
  R3Vector operator- (const R3Vector &v1, const R3Vector &v2);
  double operator* (const R3Vector &v1, const R3Vector &v2);
  R3Vector operator* (double x, const R3Vector &v);
  R3Vector operator* (const R3Vector &v, double x);
  R3Vector operator/ (const R3Vector &v1, double x);
  R3Vector operator% (const R3Vector &v1, const R3Vector &v2);


} // GerPh


#endif
